/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.descriptorWizard;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.api.errors.InvalidRemoteException;
import org.eclipse.jgit.api.errors.TransportException;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import com.fasterxml.jackson.core.JsonProcessingException;

import eu.DECIDEh2020.appManager.AppDescriptionHelper;
import eu.DECIDEh2020.appManager.AppManager;
import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.persistence.GitHandler;
import eu.DECIDEh2020.appManager.persistence.GitHandler.Repository;
import eu.DECIDEh2020.architect.plugin.natures.ProjectNature;

public class NewProjectSupport {
	/**
	 * Support class for the NewProjectWizard
	 * 
	 * For this marvelous project we need to: - create the default Eclipse project -
	 * add the custom project nature - create the folder structure
	 *
	 * @param projectName
	 * @param location
	 * @param natureId
	 * @return
	 */
	public static IProject createProject(String projectName, URI location, IProgressMonitor monitor, Repository repo,
			NewProjectWizard wizard) throws CoreException {
		Assert.isNotNull(projectName);
		Assert.isTrue(projectName.trim().length() > 0);

		GitHandler githandler = new GitHandler(repo);

		File dir;
		if (location == null) {
			dir = new File(ResourcesPlugin.getWorkspace().getRoot().getLocation() + File.separator + projectName);
		} else {
			dir = new File(location + File.separator + projectName);
		}
/*
 * repo is not local & not new: clone from remote
 * repo is local & not new: open on filesystem
 * repo is not local & new: create new one on remote? -> TODO: not yet possible
 * repo is local & new: create new on filesystem
 */
		if (!repo.isLocalRepo() && !repo.isNewRepo()) {
			try {
				githandler.cloneRepositoryTo(dir.toPath());
				// Note: the call() returns an opened repository already which needs to be
				// closed to avoid file handle leaks!
				System.out.println("Created repository: " + githandler.getRepository().getPath());
			} catch (InvalidRemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (TransportException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (GitAPIException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else if (repo.isLocalRepo() && repo.isNewRepo()) {

			try {
				githandler.createRepositoryIn(dir.toPath());
			} catch (IllegalStateException | GitAPIException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		} else if (!repo.isLocalRepo() && repo.isNewRepo()) {
			// TODO create remote repo
		}

		IProject project = createBaseProject(projectName, location);
		try {
			if (repo.isNewRepo()) {
				createFile(project, monitor, wizard);
			} else {
				IFile decide = project.getFile(new Path("DECIDE.json"));
				if (decide.exists()) {
					openFile(decide, monitor, wizard);
				}
			}
			//Add nature to Project so symbol is shown & builder applies
			addNature(project);

		} catch (CoreException e) {
			e.printStackTrace();
			project = null;
		}

		return project;
	}

	public static AppManager openAppManager() {

		return null;

	}

	private static void createFile(IProject project, IProgressMonitor monitor, NewProjectWizard wizard)
			throws CoreException {
		// create a sample file
		final String fileName = "DECIDE.json";
		monitor.beginTask("Creating " + fileName, 2);

		final IFile file = project.getFile(new Path(fileName));
		try {
			InputStream stream = openContentStream(wizard.getProject());
			if (file.exists()) {
				file.setContents(stream, true, true, monitor);
			} else {
				file.create(stream, true, monitor);
			}
			stream.close();
		} catch (IOException e) {
		}
		monitor.worked(1);
		openFile(file, monitor, wizard);

	}

	private static void openFile(IFile file, IProgressMonitor monitor, NewProjectWizard wizard) {
		monitor.setTaskName("Opening file for editing...");
		wizard.getShell().getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				try {
					IDE.openEditor(page, file, true);
				} catch (PartInitException e) {
				}
			}
		});
		monitor.worked(1);
	}

	private static InputStream openContentStream(AppDescription project) throws CoreException {

		String contents = "";
		try {
			AppDescriptionHelper adh = AppDescriptionHelper.create(project);
			contents = adh.getJson();
		} catch (JsonProcessingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throwCoreException("Error reading user generated JSON");
		}
		return new ByteArrayInputStream(contents.getBytes());
	}

	private static void throwCoreException(String message) throws CoreException {
		IStatus status = new Status(IStatus.ERROR, "eu.DECIDEh2020.architect.plugin", IStatus.OK, message, null);
		throw new CoreException(status);
	}

	/**
	 * Just do the basics: create a basic project.
	 *
	 * @param location
	 * @param projectName
	 */
	private static IProject createBaseProject(String projectName, URI location) {
		// it is acceptable to use the ResourcesPlugin class
		IProject newProject = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);

		if (!newProject.exists()) {
			URI projectLocation = location;
			IProjectDescription desc = newProject.getWorkspace().newProjectDescription(newProject.getName());
			if (location != null && ResourcesPlugin.getWorkspace().getRoot().getLocationURI().equals(location)) {
				projectLocation = null;
			}

			desc.setLocationURI(projectLocation);
			try {
				newProject.create(desc, null);
				if (!newProject.isOpen()) {
					newProject.open(null);
				}
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}

		return newProject;
	}

	private static void createFolder(IFolder folder) throws CoreException {
		IContainer parent = folder.getParent();
		if (parent instanceof IFolder) {
			createFolder((IFolder) parent);
		}
		if (!folder.exists()) {
			folder.create(false, true, null);
		}
	}

	/**
	 * Create a folder structure with a parent root, overlay, and a few child
	 * folders.
	 *
	 * @param newProject
	 * @param paths
	 * @throws CoreException
	 */
	private static void addToProjectStructure(IProject newProject, String[] paths) throws CoreException {
		for (String path : paths) {
			IFolder etcFolders = newProject.getFolder(path);
			createFolder(etcFolders);
		}
	}

	private static void addNature(IProject project) throws CoreException {
		if (!project.hasNature(ProjectNature.NATURE_ID)) {
			IProjectDescription description = project.getDescription();
			String[] prevNatures = description.getNatureIds();
			String[] newNatures = new String[prevNatures.length + 1];
			System.arraycopy(prevNatures, 0, newNatures, 0, prevNatures.length);
			newNatures[prevNatures.length] = ProjectNature.NATURE_ID;
			description.setNatureIds(newNatures);

			IProgressMonitor monitor = null;
			project.setDescription(description, monitor);
		}
	}

}
