/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.architect.patterns;

import java.io.InputStream;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ResIterator;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.Statement;
import org.apache.jena.sparql.function.library.leviathan.e;
import org.apache.jena.vocabulary.DCTerms;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;
import org.testng.asserts.SoftAssert;

import eu.DECIDEh2020.architect.cloudpatterns.DECIDEPatterns;
import eu.DECIDEh2020.architect.cloudpatterns.DP;
import eu.DECIDEh2020.architect.cloudpatterns.Patterns;

public class PatternsTest {

	private Patterns patterns;
	 private SoftAssert softAssert = new SoftAssert();

	@BeforeTest
	void initPatterns() {
		patterns = Patterns.createDefaultPatterns();
		assert patterns.getModel() != null : "Expected initial patterns model.";
	}

	@Test
	void listPatterns() {
		List<Resource> patternList = patterns.listPatterns();
		assert patternList.size() == 23 : "Expected to get a list with 23 elements. But got " + patternList.size();
	}

	@Test
	void getPattern() {
		Resource pattern = patterns.getPattern(DECIDEPatterns.BASE + "patterns/loose-coupling");
		assert pattern != null : "Expected to get loose coupling pattern.";
		assert pattern.getProperty(DCTerms.title).getString()
				.equals("Loose Coupling") : "Expected 'Loose Coupling' in title";
	}

	@Test
	void getIcon() {
		Resource pattern = patterns.getPattern(DECIDEPatterns.BASE + "patterns/stateless-component");
		assert pattern != null : "Expected stateless component pattern.";
		Resource icon = pattern.getProperty(DP.icon).getResource();
		assert icon != null : "Expected icon property";
		InputStream stream = patterns.getIcon(icon);
		assert stream != null : "Expected icon as input stream.";
	}

	@Test
	void getSolution() {
		Resource pattern = patterns.getPattern(DECIDEPatterns.BASE + "patterns/stateless-component");
		assert pattern != null : "Expected stateless component pattern.";
		assert !pattern.getProperty(DP.solution).getString().isEmpty() : "Expected non empty string.";
	}

	@Test
	void testCategories() {
		List<Resource> patternList = patterns.getPatternsByCategory(patterns.getModel().createResource(":dummy"));
		assert patternList == null : "Expected null list";
		patternList = patterns.getPatternsByCategory(DECIDEPatterns.fundamentalCategory);
		assert patternList.size() == 7 : "Expected list of patterns with 7 elements.";
		patternList = patterns.getPatternsByCategory(DECIDEPatterns.optimizationCategory);
		assert patternList.size() == 5 : "Expected empty list of 5 patterns.";
		patternList = patterns.getPatternsByCategory(DECIDEPatterns.developmentCategory);
		assert patternList.size() == 5 : "Expected list of patterns with two elements.";
	}

	@Test
	void listNFRs() {
		List<Resource> patternList = patterns.listNFRs();
		assert patternList.size() == 5 : "Expected to get a list with 5 elements.";
	}

	@Test
	void testInference() {
		List<Resource> patternList = patterns.inferPatterns(Collections.singletonList(DECIDEPatterns.availability));
		assert patternList.size() == 11 : "Expected to get a list with 3 elements. Got " + patternList.size();
	}

	@Test
	void testBadInference() {
		List<Resource> patternList = patterns.inferBadPatterns(Collections.singletonList(DECIDEPatterns.scalability));
		assert patternList.size() == 0 : "Expected to get a list with 0 elements.";
	}

	@Test
	void testHasImpactOn() {
		List<Resource> patternList = patterns.listPatterns();
		assert patternList != null : "Expected to get a list of 23 patterns";
		Iterator<Resource> iter = patternList.iterator();
		
		while (iter.hasNext()) {
			Resource pattern = iter.next();
			assert pattern != null : "Expected a pattern";
			Statement impact = pattern.getProperty(DP.hasPositiveImpactOn);
			softAssert.assertNotNull(impact,pattern.getLocalName().toString());
		}

		softAssert.assertAll();
	}

}
